var Record = function(value, expire) {
    this.value = value;
    this.expire = expire;
    this.lastModified = new Date();
    this.count = 0;
};

var Cache = function(max, expire) {
    this.max = max || 10 * 1024 * 1024;
    this.expire = expire || 30 * 60 * 1000;
    this.__queue__ = {};
    this.__cache__ = {};
};

Cache.prototype.set = function(key, value, expire) {
    var __cache__ = this.__cache__;

    var tem;
    var count = 0;

    if (!__cache__[key]) {
        tem = __cache__[key] = new Record(value, expire || this.expire);
    } else {
        tem = __cache__[key];
        tem.value = value;
        tem.lastModified = new Date();
        tem.expire = expire || tem.expire;
        count = tem.count;
        this.queueRemove(count, key);
        count += 1;
    }

    tem.count = count;
    this.queueSet(count, key);
};

Cache.prototype.get = function(key) {
    var __cache__ = this.__cache__;
    var tem = __cache__[key];

    if (!tem) return;
    if (tem.expire && (new Date() - tem.lastModified > tem.expire)) {
        this.remove(key);
        return;
    }
    var count = tem.count;
    this.queueRemove(count, key);
    count += 1;
    tem.count = count;
    this.queueSet(count, key);
    return tem.value;
};

Cache.prototype.remove = function(key) {
    var __cache__ = this.__cache__;
    if(!__cache__[key]) return;
    var count = __cache__[key].count;
    __cache__[key] = undefined;
    this.queueRemove(count, key);
};

Cache.prototype.queueSet = function(pos, key) {
    var pos = this.__queue__[pos];
    if (!pos) {
        pos = {};
    }
    pos[key] = true;
};

Cache.prototype.queueRemove = function(pos, key) {
    var pos = this.__queue__[pos];
    if(!pos) return;
    pos[key] = undefined;
};

