var cluster = require('cluster');

/* ========================================================
    Manager
======================================================== */

var Manager = function() {

    var self = this;
    
    // 初始化共享内存
    self.__sharedMemory__ = {};
        
    // 监听并处理来自worker的请求
    cluster.on('online', function(worker) {
        worker.on('message', function(data) {
            if (!data.isSharedMemoryMessage) return;
            self.handle(data);
        });
    });
};

Manager.prototype.handle = function(data) {
    var self = this;
    var value = this[data.method](data);

    var msg = {
        isSharedMemoryMessage: true,
        id: data.id,
        uuid: data.uuid,
        value: value
    };

    if(cluster.isMaster) {
        cluster.workers[data.id].send(msg);
        return;
    }

    process.send(msg);

};

Manager.prototype.set = function(data) {
    this.__sharedMemory__[data.key] = data.value;
    return 'OK';
};

Manager.prototype.get = function(data) {
    return this.__sharedMemory__[data.key];
};


/* ========================================================
    User
======================================================== */

var User = function() {
    var self = this;
    self.__uuid__ = 0;
    self.__getCallbacks__ = {};
    process.on('message', function(data) {
        var cb = self.__getCallbacks__[data.uuid];
        if (cb && typeof cb == 'function') {
            cb(data.value)
        }
        self.__getCallbacks__[data.uuid] = undefined;
    });
};

User.prototype.uuid = function() {
    //var i = this.__uuid__;
    //return this.__uuid__ = i = (i > 65535 ? 0 : i + 1);
    return this.__uuid__++;
};

User.prototype.set = function(key, value, callback) {
    this.handle('set', key, value, callback);
};

User.prototype.remove = function(key, callback) {
    this.handle('set', key, undefined, callback);
};

// eg. callback = function(data) {};
User.prototype.get = function(key, callback) {
    this.handle('get', key, null, callback);
};

User.prototype.handle = function(method, key, value, callback) {

    var self = this;
    var uuid = self.uuid();

    process.send({
        isSharedMemoryMessage: true,
        method: method,
        id: cluster.worker.id,
        uuid: uuid,
        key: key,
        value: value
    });

    self.__getCallbacks__[uuid] = callback;

};


/* ========================================================
    Init
======================================================== */

function init(config) {

    /*
        Manager(cluster master) <--> User(cluster worker)
    */        

    if (cluster.isMaster) {
        return new Manager();
    }

    return new User;

}

exports.init = init;

